# Fotbalový stadion

Nagyerdő je čtvercový les ve městě Debrecín, který si můžeme představit jako mřížku $N \times N$ polí.
Řádky jsou očíslované od $0$ do $N - 1$ ze severu na jih a 
sloupce jsou očíslované od $0$ do $N - 1$ ze západu na východ.
Buňku v řádku $r$ a sloupci $c$ značíme pole $(r, c)$ 

V lese je každé pole **prázdné** nebo obsahuje **strom**. Alespoň jedno pole v lese je prázdné.

DVSC, známý sportovní klub Debrecína plánuje postavit v lese nový fotbalový stadion.
Stadion rozměru $s$ (kde $s \ge 1$) je množna $s$ *po dvou různých prázdných* polí $(r_0, c_0), \ldots, (r_{s-1}, c_{s-1})$.
Formálně:
- každé pole $(r_i, c_i)$ je prázdné
- každá dvě pole $(r_i, c_i), (r_j, c_j)$ platí, že se liší v alespoň jedné souřadnici ($r_i \neq r_j \lor c_i \neq c_j$).

Fotbal se hraje kopaním do míče na polích stadionu.
**Rovný kop** definujeme jako jednu ze dvou akcí:
* Přemístění míče z pole $(r, a)$ na pole $(r, b)$ ($0 \le r,a,b \lt N, a \ne b$), kde stadion obsahuje všechny pole mezi $(r, a)$ a $(r, b)$. Formálně:
	* Když $a < b$, pak stadion musí obsahovat pole $(r, k)$, pro každé $k$ takové, že $a \leq k \leq b$
	* Když $a > b$, pak stadion musí obsahovat pole $(r, k)$, pro každé $k$ takové, že $b \leq k \leq a$
* Přemístění míče z pole $(a, c)$ na pole $(b, c)$ ($0 \le c,a,b \lt N, a \ne b$), kde stadion obsahuje všechny pole mezi $(a, c)$ a $(b, c)$. Formálně:
	* Když $a < b$, pak stadion musí obsahovat pole $(k, c)$, pro každé $k$ takové, že $a \leq k \leq b$
	* Když $a > b$, pak stadion musí obsahovat pole $(k, c)$, pro každé $k$ takové, že $b \leq k \leq a$

Stadion je **řádný** pokud je možné přemístit míč mezi libovolnými dvěma poli s nejvýše $2$ rovnými kopy.
Stadion velikost $1$ je řádný.

Například, uvažujte les velikost $N = 5$, s poli $(1,0)$ a $(4,2)$ obsahujícími stromy a ostatními poli prázdnými.
Obrázek níže ukazuje tři možné stadiony. Pole se stromy jsou tmavé, pole zabrané stadionem jsou pruhované.

![Stadion](soccer-regular.png "650")

Levý stadion je řádný. Stadion uprostřed řádný není, protože jsou zapotřebí $3$ rovné kopy z pole $(4,1)$ na $(4,3)$.
Stadion vpravo řádný není, protože není možné přemístit míč mezi buňkami $(3,0)$ a $(1,3)$ s použitím rovných kopů.

Sportovní klub chce postavit co největší stadion.
Vaším úkolem je najít největší možné $s$ takové, že v lese existuje řádný stadion velikosti $s$.

## Implementační detaily

Měli byste implementovat následující funkci:
```
int biggest_stadium(int N, int[][] F)
```

* $N$: velikost lesa.
* $F$: seznam délky $N$ obsahující seznamy délky $N$, popisující pole v lese. Pro každé $r$ a $c$ takové, že $0 \le r \lt N$ a $0 \le c \lt N$, $F[r][c] = 0$ znamená, že pole $(r, c)$ je prázdné, a $F[r][c] = 1$ znamená pole obsahující strom.
* Funkce má vracet maximální velikost řádného stadionu, který lze v lese postavit.
* Funkce je zavolána právě jednou pro každý vstup.

## Příklad

Uvažte následující volání:

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

V tomto případě, les je vyznačen vlevo a řádný stadion s velikostí $20$ je vyznačen vpravo:

![Example](soccer-example.png "400")

Protože neexistuje žádný stadion s velikostí alespoň $21$, funkce by měla vrátit $20$.

## Omezení

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (pro každé $i$ a $j$ takové, že $0 \le i \lt N$ a $0 \le j \lt N$)
* Existuje alespoň jedno prázdné pole v lese. (Tedy $F[i][j] = 0$ pro alespoň jedno $0 \le i, j < N$.)

## Podúlohy

1. (6 bodů) Nejvýše jedno pole obsahuje strom.
2. (8 bodů) $N \le 3$
3. (22 bodů) $N \le 7$ 
4. (18 bodů) $N \le 30$ 
5. (16 bodů) $N \le 500$
6. (30 bodů) Bez dalších omezení.

V každé podúloze lze získat 25% bodů, pokud program správně posoudí, zda množina *všech* prázdných polí jsou řádný stadion.

Přesněji, pro každou podúlohu, ve které množina všech prázdných polí je řádný stadion, vaše řešení:
* Dostane plný počet bodů, pokud vrátí správnou odpověď (což je velikost množiny všech prázdných polí).
* Jinak 0 bodů.

Pro každou podúlohu, ve které množina všech prázdných polí *není* řádný stadion, vaše řešení:
* Dostane plný počet bodů, pokud vrátí správnou odpověď.
* Dostane 0 bodů, pokud vrátí velikost množiny všech prázdných polí.
* Dostane 25% bodů, pokud vrátí jakoukoliv jinou hodnotu.

Skóre za každou podúlohu je minimum z bodů za každý vstup v dané podúloze.

## Ukázkový Grader

Ukázkový grader čte vstup v následujícím formátu: 

* řádek $1$: $N$
* řádek $2 + i$  ($0 \le i \lt N$):  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
Ukázkový grader vypisuje odpověď v následujícím formátu:
 
* řádek $1$: vrácená hodnota `biggest_stadium`

