# Nogometni stadion

Maksimir je park kvadratnog oblika koji se nalazi u Zagrebu, sjeverno od lokacije trenutnog stadiona zagrebačkih miljenika, koji se može prikazati kao $N \time N$ tablica.
Stupci tablice označeni su brojevima od $0$ do $N - 1$ od zapada prema istoku, dok su retci označeni brojevima od $0$ do $N - 1$ od sjevera prema jugu.
Polje koje se nalazi u $r$-tom retku i $c$-tom stupcu označivat ćemo kao $(r, c)$.

U šumi, svako polje je ili **prazno** ili se na njemu nalazi **slon**.
Barem jedno polje Maksimira je prazno.

**Gradski** nogometni klub Dinamo Zagreb, novcem poreznih obveznika planira izgraditi novi stadion kako bi igrači tijekom
utakmica mogli uživati u pogledu na još veći broj praznih sjedala.
Stadion veličine $s$ (gdje je $s \ge 1$) je skup $s$ *različitih praznih* polja $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$.
Točnije, za svaki $i$ između $0$ i $s - 1$, uključivo, polje $(r_i, c_i)$ je prazno, i za svaki $j$ takav da je $i \lt j \lt s$, vrijedi $r_I \neq r_j$ ili $c_i \neq c_j$. 

Nogomet se igra loptom koja stalno leti po poljima stadiona.
**Ravan udarac**, kakvim je nekada Luka Modrić neka zabavljao zločeste plave dečke, može biti jedna od sljedeće dvije akcije:

* Pomicanje lopte s polja $(r, a)$ u polja $(r, b)$ ($0 \le r, a, b, \lt N, a \ne b$), gdje stadion sadrži *sva* polja između polja $(r, a)$ i $(r, b)$ unutar $r$-tog retka. Točnije
 - ako je $a \lt b$, tada stadion mora sadržavati sva polja $(r, k)$ za svaki $k$ takav da je $a \le k \le b$,
 - ako je $a \gt b$, tada stadion mora sadržavati sva polja $(r, k)$ za svaki $k$ takav da je $b \le k \le a$,
 * Pomicanje lopte iz polja $(a, c)$ do polja $(b, c)$ ($0 \le c, a, b, \lt N, a \ne b$), gdje stadion sadrži *sva* polja između polja $(c, a)$ i $(c, b)$ unutar $c$-tog stupca. Točnije
 - ako je $a \lt b$, tada stadion mora sadržavati sva polja $(k,c)$ za svaki $k$ takav da je $a \le k \le b$,
 - ako je $a \gt b$, tada stadion mora sadržavati sva polja $(k,c)$ za svaki $k$ takav da je $b \le k \le a$,

No, Modrić se toga ionako ne sjeća.

Stadion je **regularan** ako je moguće loptu iz bilo kojeg polja prebaciti do bilo kojeg drugog polja u najviše dva ravna udarca.
Mamić, u intimnom razgovoru sa svojim lokalnim stručnjacima u svom trenutnom prebivalištu, zaključio je da je to svojstvo ključno za prolazak Dinama u **FINALE LIGE PRVAKA**.
Primijetite da je stadion veličine $1$ uvijek regularan.

Primjerice, promatrajmo Maksimir veličine $N = 5$, gdje polja $(1, 0)$ i $(4, 2)$ sadrže slonove dok su ostala polja prazna.
Slika prikazuje tri moguća stadiona. Zatamnjena polja su slonovi dok su iscrtana polja dio stadiona.

![Stadiums](soccer-regular.png "650")

Prvi stadion je regularan. Ali, drugi stadion nije regularan, jer su potrebna barem tri ravna udarca kako bi se lopta prebacila s polja $(4, 1)$ u polje $(4, 3)$, Treći stadion također nije regularan, jer je nemoguće loptu prebaciti s polja $(3, 0)$ u polje $(1, 3)$ koristeći ravne udarce.

<s>Mamić</s> Dinamo želi izgraditi što veći regularan stadion.
Vaš je zadatak odrediti najveću vrijednost $s$ takvu da unutar Maksimira postoji regularan stadion veličine $s$.

## Implementacijski detalji

Morate implementirati sljedeću funkciju

```
int biggest_stadium(int N, int[][] F)
```

* $N$: veličina Maksimira
* $F$: niz duljine $N$ koji sadrži nizove duljine $N$, koji opisuju polja Maksimira. Za svaki $r$ i $c$ takav da je $0 \le r \lt N$ i $0 \le c \lt N$, $F[r][c] = 0$ označava da polje $(r, c)$ prazno, u suprotnom je $F[r][c] = 1$ te se na polju nalazi slon.
* Funkcija mora vratiti veličinu najvećeg Mamićabilnog stadiona kojij se može sagraditi ana Maksimru
* Funkcija će biti pozvana točno jednom po primjeru

## Primjer

Promatrajmo sljedeći poziv:

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

U ovom primjeru, Maksimir je prikazan s lijeve starne te regularan stadion veličine $20$ prikazan je s desne strane sljedeće slike:

![Example](soccer-example.png "400")

Kako ne postoji regularan stadion veličine veće od $21$, funkcija treba vratiti $20$.

## Ograničenja

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (za svaki $i$ te $j$ takav da je $0 \le i \lt N$ i $0 \le j \lt N$)
* Postoji barem jedno prazno polje na Maksimiru. Drugim riječima, $F[i][j] = 0$ za neke $0 \le i \lt N$ te $0 \le j \lt N$.

## Podzadaci

1. (6 bodova) Najviše jedno polje sadrži slona.
2. (8 bodova) $N \le 3$
3. (22 boda) $N \le 7$ 
4. (18 bodova) $N \le 30$ 
5. (16 bodova) $N \le 500$
6. (30 bodova) Nema dodatnih ograničenja.

U svakom podzadatku, moguće je osvojiti djelomične bodove
ako vaše rješenje točno odredi je li skup *svih* praznih polja regularan stadion.

Točnije, ako za svaki primjer u kojem je podskup svih praznih polja regularan stadion, vaše rješenje
* osvaja sve bodove ako vrati točan odgovor (veličinu skupa koji se sastoji od svih praznih polja).
* osvaja nula bodova inače.

Za svaki primjer u kojem skup svih praznih polja *nije* regularan stadion, vaše rješenje:
* osvaja sve bodove ako vrati točan odgovor
* osvaja nula bodova ako vrati veličinu skupa svih praznih polja
* osvaja 25% bodova ako vrati bilo koju drugu vrijednost

Broj bodova svakog podzadatka najmanji je broj bodova primjera u podzadatku.

## Probni ocjenjivač

Probni ocjenjivač učitava ulaz u sljedećem obliku:

* $1.$ redak : $N$
* $(2 + i)$-ti redak : ($0 \le i \lt N$):  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
Probni ocjenjivač ispisuje odgovor u sljedećem obliku
 * $1.$ redak : vrijednost koju je vratila funkcija `biggest_stadium`
 
