# Soccer Stadium

Нагуэрдо это лес квадратной формы, расположенный в городе Дебрецен, который может быть представлен сеткой $N \times N$ клеток. Строки сетки пронумерованы от $0$ до $N - 1$ с севера на юг, столбцы пронумерованы от $0$ до $N - 1$ с запада на восток. Обозначим клетку в строке $r$ и столбце $c$ сетки как клетку $(r, c)$. 

В лесу, каждая клетка или **пуста**, или содержит **дерево**. По крайней мере одна из клеток в лесу пуста.

DVSC, известный городской спортивный клуб, планирует построить новый футбольный стадион в лесу. Стадион размера $s$ (где $s \ge 1$) представляет собой множество из $s$ *различных пустых* клеток  $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$. 
Формально это означает, что:
- для каждого $i$ от $0$ до $s - 1$ включительно, клетка $(r_i, c_i)$ пуста,
- для всех $i, j$, таких что $0 \le i \lt j \lt s$, выполнено хотя бы одно из условий $r_i \neq r_j$ или $c_i \neq c_j$.

В футбол играют мячом, который будет перемещаться по клеткам стадиона.
**Прямой удар** определен как одно из двух действий:
* Переместить мяч из клетки $(r,a)$ в клетку $(r,b)$ ($0 \le r,a,b \lt N, a \ne b$), при этом стадион должен содержать *все* клетки от $(r,a)$ до $(r,b)$ в строке $r$. Формально,
 - если $a \lt b$, то стадион должен содержать клетки $(r,k)$ для всех $k$, таких что $a \le k \le b$,
 - если $a \gt b$, то стадион должен содержать клетки $(r,k)$ для всех $k$, таких что $b \le k \le a$.
* Переместить мяч из клетки $(a,c)$ в клетку $(b,c)$ ($0 \le c,a,b \lt N, a \ne b$), при этом стадион должен содержать *все* клетки от $(a,c)$ до $(b,c)$ в столбце $c$. Формально,
 - если $a \lt b$, то стадион должен содержать клетки $(k,c)$ для всех $k$, таких что $a \le k \le b$,
 - если $a \gt b$, то стадион должен содержать клетки  $(k,c)$ для всех $k$, таких что $b \le k \le a$.

Назоваем стадион **регулярным**, если существует способ переместить мяч из любой клетки, принадлежащей стадиону, в другую клетку, принадлежащую стадиону, не более чем за $2$ прямых удара.
Обратите внимание, что стадион размера $1$ всегда регулярен.

Например, рассмотрим лес размера $N = 5$, где клетки $(1,0)$ и $(4,2)$ содержат деревья, а остальные клетки пусты.
 Ниже приведены примерых трех возможных стадионов. Клетки с деревьями покрашены в черный, а клетки, принадлежащие стадиону, заштрихованы.
 
![Stadiums](soccer-regular.png "650")

Стадион слева регулярен. Однако, стадион в центре не регулярен, поскольку необходимо как минимум $3$ прямых удара чтобы переместить мяч из клетки $(4,1)$ в клетку $(4,3)$. Стадион справа также не регулярен, поскольку невозможно переместить мяч из клетки $(3,0)$ в клетку $(1,3)$ используя прямые удары.

Спортивный клуб хочет построить регулярный стадион максимального размера.
Ваша задача найти максимальное значение $s$, такое что в лесу можно разместить регулярный стадион размера $s$.

## Implementation Details

Вы должны реализовать следующую функцию.

```
int biggest_stadium(int N, int[][] F)
```

* $N$: размер леса.
* $F$: массив длины $N$, содержащий массивы длины $N$, описывающие клетки в лесу. Для каждого $r$ и $c$, таких что $0 \le r \lt N$ и $0 \le c \lt N$, $F[r][c] = 0$ обозначает, что клетка $(r, c)$ пуста, и $F[r][c] = 1$ обозначает, что она содержит дерево.
* Эта функция должна вернуть максимальный размер регулярного стадиона, который может быть построен в лесу.
* Эта функция вызывается ровно один раз для каждого теста.

## Example

Рассмотрим следующий вызов:

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

На картинке ниже, слева изображен лес, справа изображен регулярный стадион размера $20$:

![Example](soccer-example.png "400")

Так как не существует регулярного стадиона размера хотя бы $21$, функция должна вернуть $20$.

## Constraints

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (для каждого $i$ и $j$, таких что $0 \le i \lt N$ и $0 \le j \lt N$)
* Существует по крайней мере одна пустая клетка в лесу. Другими словами, $F[i][j] = 0$ для некоторых $0 \le i \lt N$ и $0 \le j \lt N$.

## Subtasks

1. (6 баллов) Есть не более одной клетки, содержащей дерево.
2. (8 баллов) $N \le 3$
3. (22 балла) $N \le 7$ 
4. (18 баллов) $N \le 30$ 
5. (16 баллов) $N \le 500$
6. (30 баллов) Нет дополнительных ограничений.

В каждой подзадаче, вы можете получить 25% баллов этой подзадачи, если ваша программа корректно определяет, является ли множество *всех* пустых клеток регулярным стадионом.

А именно, для каждого теста, в котором множество всех пустых клеток образует регулярный стадион, ваше решение:
* получит полный балл, если оно возвращает правильный ответ (который равен размеру множества всех пустых клеток).
* получит 0 баллов в противном случае.

Для каждого теста, в котором множество всех пустых клеток *не* образует регулярный стадион, ваше решение:
* получит полный балл, если оно возвращает правильный ответ.
* получит 0 баллов, если оно возвращает размер множества из всех пустых клеток.
* получит 25% баллов, если оно возвращает любое другое значение.

Баллы за каждую подзадачу определяются как минимум баллов по всем тестам этой подзадачи.

## Sample Grader

Грейдер читает данные в следующем формате: 
 
* строка $1$: $N$
* строка $2 + i$  ($0 \le i \lt N$):  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
Грейдер выводит ваш ответ в следующем формате:
* строка $1$: возвращаемое значение `biggest_stadium`
