# Soccer Stadium

Nagyerdő er ferningslaga skógur, staðsettur í borginni Debrecen, sem má setja upp í líkan af $N \times N$ reitum.
Dálkarnir í líkaninu er númeraðir frá $0$ til $N-1$ frá vestri til austur og raðirnar eru númeraðar frá $0$ til $N-1$ frá norðri til suðurs.
Við vísum í reitinn sem er staðsettur í röð $r$ og dálki $c$ í líkaninu sem reit $(r, c)$.

Í skóginum  gildir fyrir sérhvern reit að hann sé **tómur** eða inniheldur **tré**.
Í minnsta lagi einn reitur í skóginum er tómur.

Fræga fótboltaliðið í borginni DVSC er að skipuleggja það að byggja nýjan leikvang í skóginum.
Leikvangur af stærð $s$ (þar sem $s \ge 1$) er mengi af $s$ *mismunandi tómum* reitum $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$.
Formleg skilgreining er því:
- fyrir sérhvert $i$ frá $0$ til $s-1$, með báðum með töldum, þá er reitur $(r_i, c_i)$ tómur,
- fyrir sérhvert $i, j$, þannig að $0 \le i \lt j \lt s$, þá gildir $r_i \neq r_j$ eða $c_i \neq c_j$ eða bæði.

Fótbilti er spilaður með bolta sem hreyfist á milli reita í leikvanginum.
**Beint spark** er skilgreint sem annaðhvort af eftirfarandi tvem aðgerðum:
* Færa boltann frá reit $(r, a)$ í reit $(r, b)$ ($0 \le r,a,b \lt N, a \ne b$), þar sem leikvangurinn inniheldur *alla* reiti á milli reitanna $(r,a)$ og $(r,b)$ í röð $r$. Formlega,
- ef $a \lt b$ þá skal leikvangurinn innihalda reit $(r, k)$ fyrir sérhvert $k$ þannig að $a \le k \le b$,
- ef $ a \gt b$ þá skal leikvangurinn iniihalda reit $(r,k)$ fyrir sérhvert $k$ þannig að $b \le k \le a$.
* Færa boltann frá reit $(a,c)$ til reits $(b,c)$ ($0 \le c,a,b \lt N, a \ne b$), þar sem leikvangurinn inniheldur *alla* reitir á milli reitanna $(a,c)$ og $(b,c)$ í röð $r$. Formlega,
- ef $a \lt b$ þá skal leikvangurinn innihalda reit $(k, c)$ fyrir sérhvert $k$ þannig að $a \le k \le b$,
- ef $ a \gt b$ þá skal leikvangurinn iniihalda reit $(k,c)$ fyrir sérhvert $k$ þannig að $b \le k \le a$.

Leikvangur er **reglulegur** ef hægt er að færa boltan frá sérhverjum reit í leikvanginum
til sérhvers annars reits í leikvanginum í mesta lagi $2$ beinum spörkum.
Takið eftir því að allir leikvangar af stærð $1$ eru reglulegir.

Skoðum eftirfarandi dæmi, gerum ráð fyrir skóg af stærð $N = 5$, þar sem reitir $(1, 0)$ og $(4, 2)$ innihalda tré og allir aðrir reitir er tómir.
Myndin hér að neða sýnir þrjár mögulegar staðsetningar af leikvöngum.Svörtu reitirnir tákna tré og blástrípuðu reitirnir innihalda hluta af leikvangnum.
![Stadiums](soccer-regular.png "650")

Leikvangurinn til vinstri er reglulegur. Hinsvegar er leikvangurinn í miðjunni ekki reglulegur,
þar sem það þarf að minnsta kost $3$ bein spörk til að hreyfa bolta frá reit
$(4,1)$ í reit $(4,3)$. Leikvangurinn til hægri er einnig ekki reglulegur, þar sem það er ómögulegt
að hreyfa boltan frá reit $(3,0)$ í reit $(1,3)$ með því að nota bein spörk.

Fótboltaliðið vill byggja leikvang sem er eins stór og hægt er.
Þitt verkefni er að finna stærsta gildið $s$, þannig að það sé til reglulegur leikvangur af stærð $s$ í skóginum.

## Útfærsluatriði

Þú skalt að útfæra eftirfarandi fall.

```
int biggest_stadium(int N, int[][] F)
```

* $N$: stærðin á skóginum.
* $F$: fylki af lengd $N$ sem inniheldur fylki (í fleirtölu) af lengd $N$, sem lýsa reitunum í skóginum. Fyrir sérhvert $r$ og $c$ þannig að $0 \le r \lt N$ and $0 \le c \lt N$, $F[r][c] = 0$
þýðir að reitur $(r,c)$ sé tómur og  $F[r][c] = 1$ þýðir að reitur inniheldur tré.
* Þetta fall skal skila frá sér stærstu stærð af reglulegum leikvangi sem hægt er að byggja í skóginum.
* Þetta fall er kallað á nákvæmlega einu sinni í hverju prufutilveki.

## Sýnidæmi

Íhugið eftirfarandi kall:

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

Í þessu sýnidæmi, þá er skógurinn sýndur á vinstri og reglulegur leikvangur af stærð $20$ er sýndur á hægri í eftirfarandi mynd:

![Example](soccer-example.png "400")

Þar sem að það er engin reglulegur leikvangur af stærð $21$ eða stærra, þá skal fallið skila $20$.

## Skorður

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (fyrir sérhvert $i$ og $j$ þannig að $0 \le i \lt N$ og $0 \le j \lt N$)
* There is at least one empty cell in the forest. In other words, $F[i][j] = 0$ for some $0 \le i \lt N$ and $0 \le j \lt N$.
* Það er allavega einn tómur reitur í hverjum skógi. Með öðrum orðum, $F[i][j] = 0$ fyrir eitthvað $0 \le i \lt N$ og $0 \le j \lt N$.

## Hlutverkefni

1. (6 points) Það er í mesta lagi einn reitur sem inniheldur tré.
2. (8 points) $N \le 3$
3. (22 points) $N \le 7$ 
4. (18 points) $N \le 30$ 
5. (16 points) $N \le 500$
6. (30 points) Engir frekari skorður.

Í sérhverju hlutverkefni getur þú fengið fullt hús stiga eða hluta af heildarstigum,
ef forritið þitt dæmir rétt hvort til er mengi af *öllum* tómum reitum sem mynda reglulegan leikvang.

Nánar tiltekið, fyrir sérhvert prufutilvik þar sem mengi af öllum tómum reitum er reglulegur leikvangur, þá fær lausnin:
* full stig ef hún skilar réttu svari (sem er stærðin af menginu af öllum tómum reitum)
* 0 stig annars

Fyrir sérhvert prufutilvik þar sem mengið af öllum tómum reitum er *ekki* reglulegur leikvangur, þá fær lausnin:
* full stig ef hún skilar réttu svari.
* 0 stig ef hún skilar stærðinni á menginu af öllum tómum reitum
* 25% af stigunum ef hún skilar einhverju öðru gildi.

Stigin á hlutverkefninu eru minnstu stig sem fengin voru fyrir prufutilvikin í hlutverkefninu.

## Sýnisyfirferðarforrit

Sýnisyfirferðarforritið les inntakið á eftirfarandi sniði:
 
* lína $1$: $N$
* lína $2 + i$  ($0 \le i \lt N$):  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
Sýnisyfirferðarforritið skrifar út svarið þitt á eftirfarandi sniði:
* lína $1$: skilagildið á `biggest_stadium`
