# サッカースタジアム (Soccer Stadium)

Nagyerdő は都市 Debrecen にある正方形の形をした森であり，$N \times N$ マスのグリッドとして表現することができる．グリッドの行は北から順に $0$ から $N - 1$ までの番号が付けられており，グリッドの列は西から順に $0$ から $N - 1$ までの番号が付けられている．行 $r$ と列 $c$ が交わるマスをマス $(r, c)$ と書くことにする．

森の各マスは，**木の生えているマス** か **空マス** のどちらかである．少なくとも $1$ つのマスが空であることが保証される．

この都市の有名なスポーツクラブである DVSC は，この森に新しいサッカースタジアムを作ろうとしている．大きさ $s$ ($s \ge 1$) のサッカースタジアムとは，$s$ 個の*相異なる空マス* $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$ の集まりである．これらが相異なる空マスであるとは，形式的には，以下の条件を満たすことをいう．

- $0$ 以上 $s - 1$ 以下の各 $i$ について，マス $(r_i, c_i)$ は空マスである．
- $0 \le i \lt j \lt s$ であるような各 $i, j$ について，$r_i \neq r_j$ か $c_i \neq c_j$ のうち少なくとも一方が成り立つ．

サッカーでは，$1$ つのボールをスタジアムのマスの中のあちこちに移動させて遊ぶ．以下のいずれかに当てはまる行動を **まっすぐなキック** と呼ぶ．

* マス $(r, a)$ にあるボールをマス $(r, b)$ に移動させる ($0 \le r,a,b \lt N, a \ne b$)．ただし，行 $r$ において，マス $(r, a)$ とマス $(r, b)$ の間の*すべて*のマスがスタジアムに含まれなければならない．形式的には，
 - $a \lt b$ のとき，$a \le k \le b$ であるようなすべての $k$ について，マス $(r, k)$ がスタジアムに含まれなければならない．
 - $a \gt b$ のとき，$b \le k \le a$ であるようなすべての $k$ について，マス $(r, k)$ がスタジアムに含まれなければならない．
* マス $(a, c)$ にあるボールをマス $(b, c)$ に移動させる ($0 \le c,a,b \lt N, a \ne b$)．ただし，列 $c$ において，マス $(a, c)$ とマス $(b, c)$ の間の*すべて*のマスがスタジアムに含まれなければならない．形式的には，
 - $a \lt b$ のとき，$a \le k \le b$ であるようなすべての $k$ について，マス $(k, c)$ がスタジアムに含まれなければならない．
 - $a \gt b$ のとき，$b \le k \le a$ であるようなすべての $k$ について，マス $(k, c)$ がスタジアムに含まれなければならない．

スタジアムは，スタジアムに含まれるどのマスからどのマスへも高々 $2$ 回のまっすぐなキックでボールを移動させることができるとき，**標準的である** と言う．大きさ $1$ のスタジアムはどれも標準的であることに注意せよ．

例えば，$N = 5$ の大きさで，マス $(1,0)$ とマス $(4,2)$ に木が生えていて，残りのマスが空であるような森について考える．以下の図は，$3$ つのスタジアムの例を表している．黒マスが木の生えているマスを表し，斜線の入ったマスがスタジアムを表す．
 
![Stadiums](soccer-regular.png "650")

左の図のスタジアムは標準的である．しかし，中央の図のスタジアムは標準的でない．なぜなら，マス $(4, 1)$ からマス $(4, 3)$ にボールを移動させるのに少なくとも $3$ 回のまっすぐなキックが必要になるためである．右の図のスタジアムもまた標準的でない．なぜなら，何回かのまっすぐなキックでマス $(3, 0)$ からマス $(1, 3)$ にボールを移動させることができないためである．

このスポーツクラブは，できるだけ大きい標準的なスタジアムを作りたいと考えている．あなたの課題は，この森の中に作ることができる標準的なスタジアムの大きさ $s$ の最大値を求めることである．

## 実装の詳細

あなたは，次の関数を実装する必要がある．

```
int biggest_stadium(int N, int[][] F)
```

* $N$：森の大きさ．
* $F$：森の各マスを表す，長さ $N$ の配列を要素に持つ長さ $N$ の配列．
$0 \le r \lt N$ と $0 \le c \lt N$ を満たす組 $r, c$ について，$F[r][c] = 0$ はマス $(r, c)$ が空であることを，$F[r][c] = 1$ はマス $(r, c)$ に木が生えていることを表す．
* この関数は，この森の中に作ることができる標準的なスタジアムの大きさの最大値を返さなければならない．
* それぞれのテストケースについて，この関数はちょうど $1$ 回呼び出される．

## 例

以下の呼び出しを考える：

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

この例では，森は左の図のようになっており，右の図のように大きさ $20$ の標準的なスタジアムが存在する．

![Example](soccer-example.png "400")

大きさ $21$ 以上の標準的なスタジアムは存在しないため，この関数は $20$ を返さなければならない．

## 制約

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$（$0 \le i \lt N$，$0 \le j \lt N$）
* 森には少なくとも $1$ つの空マスが存在する．すなわち，$F[i][j] = 0$ であるような $i, j$（$0 \le i \lt N$，$0 \le j \lt N$）が存在する．

## 小課題

1. ($6$ 点) 木の生えているマスは $1$ 個以下である．
2. ($8$ 点) $N \le 3$
3. ($22$ 点) $N \le 7$ 
4. ($18$ 点) $N \le 30$ 
5. ($16$ 点) $N \le 500$
6. ($30$ 点) 追加の制約はない．

各小課題について，*すべての* 空マスからなるスタジアムが標準的であるかどうかをあなたのプログラムが正しく判定できれば，その小課題の $25$% の得点を得ることができる．

より正確には，すべての空マスからなるスタジアムが標準的であるようなテストケースについては，あなたのプログラムは以下のように評価される．

* あなたのプログラムが正しい答え（これはすべての空マスからなるスタジアムの大きさに等しい）を返したとき，$100$% の点数として評価される．
* これ以外の値を返したとき，$0$ 点として評価される．

また，すべての空マスからなるスタジアムが標準的で*ない*ようなテストケースについては，あなたのプログラムは以下のように評価される．

* あなたのプログラムが正しい答えを返したとき，$100$% の点数として評価される．
* すべての空マスからなるスタジアムの大きさを返したとき，$0$ 点として評価される．
* これら以外の値を返したとき，$25$% の点数として評価される．

各小課題の点数は，その小課題に含まれるテストケースの点数の最小値となる．

## 採点プログラムのサンプル

採点プログラムのサンプルは以下の形式で⼊⼒を読み込む．
 
* $1$ 行目：$N$
* $2 + i$ 行目（$0 \le i \lt N$）：$F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
採点プログラムのサンプルは以下の形式であなたの答えを出力する．
* $1$ 行目：`biggest_stadium` の返り値