# Futbola stadions
Nagjerdo ir kvadrātisks mežs Debrecenas pilsētā, kas var būt raksturots kā $N \times N$ rūtiņu režģis.
Režga rindas ir sanumurētas no $0$ līdz $N-1$ no ziemeļiem uz dienvidiem, un kolonnas ir sanumurētas no $0$ līdz $N-1$ no rietumiem uz austrumiem.
Rūtiņu, kas atrodas rindā $r$ un kolonnā $c$, apzīmēsim ar $(r, c)$.

Mežā katra rūtiņa ir vai nu **tukša**, vai nu satur **koku**.
Vismaz viena rūtiņa mežā ir tukša.

DVSC, slavens pilsētas futbola klubs, plāno uzbūvēt jaunu futbola stadionu šajā mežā.
Stadions ar izmēru $s$ (kur $s \ge 1$) ir $s$ *dažādu tukšu* rūtiņu kopa $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$.
Formāli, tas nozīmē, ka:
- katram $i$ no $0$ līdz $s - 1$ ieskaitot, rūtiņa $(r_i, c_i)$ ir tukša
- visiem $i, j$, kuriem $0 \le i \lt j \lt s$, izpildās vismaz viens no $r_i \neq r_j$ vai $c_i \neq c_j$.

Futbolu spēlē ar bumbu, kas tiek pārvietota stadiona rūtiņās.
**Taisns sitiens** tiek definēts kā jebkura no šīm divām darbībām:
* Pārvietot bumbu no rūtiņas $(r,a)$ rūtiņā $(r,b)$ ($0 \le r,a,b \lt N, a \ne b$), kur stadions satur *visas* rūtiņas starp rūtiņām $(r,a)$ un $(r,b)$ rindā $r$. Formāli,
 - ja $a \lt b$, tad stadionam jāsatur rūtiņa $(r,k)$ katram $k$, kuram $a \le k \le b$,
 - ja $a \gt b$, tad stadionam jāsatur rūtiņa $(r,k)$ katram $k$, kuram $b \le k \le a$.
* Pārvietot bumbu no rūtiņas $(a,c)$ rūtiņā $(b,c)$ ($0 \le c,a,b \lt N, a \ne b$), kur stadions satur *visas* rūtiņas starp rūtiņām $(a,c)$ un $(b,c)$ kolonnā $c$. Formāli,
 - ja $a \lt b$, tad stadionam jāsatur rūtiņa $(k,c)$ katram $k$, kuram $a \le k \le b$,
 - ja $a \gt b$, tad stadionam jāsatur rūtiņa $(k,c)$ katram $k$, kuram $b \le k \le a$.

Stadionu sauksim par **regulāru**, ja ir iespējams pārvietot bumbu no jebkuras stadiona rūtiņas jebkurā citā stadiona rūtiņā, izmantojot ne vairāk kā $2$ taisnus sitienus.
Ņemiet vērā, ka jebkurš stadions ar izmēru $1$ ir regulārs.

Piemēram, aplūkosim mežu ar malas garumu $N = 5$, ar kokiem rūtiņās $(1,0)$ un $(4,2)$, un visām pārējām rūtiņām tukšām.
Zīmējums zemāk attēlo trīs iespējamus stadionus. Rūtiņas ar kokiem ir tumšas, un rūtiņas, kuras satur stadions, ir strīpainas.
 
![Stadiums](soccer-regular.png "650")

Stadions pa kreisi ir regulārs. Turpretīm, stadions vidū nav regulārs, jo ir nepieciešami vismaz $3$ taisni sitieni, lai pārvietotu bumbu no rūtiņas $(4,1)$ uz $(4,3)$. Stadions pa labi arī nav regulārs, jo nav iespējams pārvietot bumbu no rūtiņas $(3,0)$ uz $(1,3)$, izmantojot tikai taisnus sitienus.

Sporta klubs vēlas uzbūvēt pēc iespējas lielāku stadionu.
Jūsu uzdevums ir atrast lielāko $s$ vērtību tādu, ka dotajā mežā var uzbūvēt izmēra $s$ stadionu.

## Implementācijas detaļas

Jums ir jāimplementē šāda procedūra.

```
int biggest_stadium(int N, int[][] F)
```

* $N$: meža izmērs.
* $F$: masīvs garumā $N$ ar masīviem garumā $N$, kas apraksta meža rūtiņas. Katriem $r$ un $c$, kuriem $0 \le r \lt N$ un $0 \le c \lt N$, $F[r][c] = 0$ nozīmē, ka rūtiņa $(r, c)$ ir tukša, un $F[r][c] = 1$ nozīmē, ka tā satur koku.
* Šai procedūrai jāatgriež lielākais regulāra stadiona izmērs, kuru var uzbūvēt mežā.
* Šī procedūra tiek izsaukta tieši vienu reizi katram testam.

## Piemērs

Aplūkosim šādu izsaukumu:

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

Šajā piemērā, mežs ir attēlots pa kreisi un regulārs stadions ar izmēru $20$ ir attēlots pa labi šajā zīmējumā:

![Example](soccer-example.png "400")

Tā kā nevar uzbūvēt regulāru stadionu ar izmēru $21$ vai vairāk, procedūrai jāatgriež $20$.

## Ierobežojumi

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (visiem $i$ un $j$, kuriem $0 \le i \lt N$ un $0 \le j \lt N$)
* Mežā ir vismaz viena tukša rūtiņa. Citiem vārdiem, $F[i][j] = 0$ kādiem $0 \le i \lt N$ un $0 \le j \lt N$.

## Apakšuzdevumi

1. (6 punkti) Ir ne vairāk par vienu rūtiņu, kas satur koku.
2. (8 punkti) $N \le 3$
3. (22 punkti) $N \le 7$ 
4. (18 punkti) $N \le 30$ 
5. (16 punkti) $N \le 500$
6. (30 punkti) Bez papildus ierobežojumiem.

Katrā apakšuzdevumā jūs varat iegūt 25% no visiem punktiem, ja jūsu programma pareizi nosaka, vai kopa ar *visām* tukšām rūtiņām ir regulārs stadions.

Precīzāk, katrā testā, kur kopa ar visām tukšām rūtiņām ir regulārs stadions, jūsu risinājums:
* iegūst pilnus punktus, ja atgriež pareizu atbildi (kas ir kopas ar visām tukšām rūtiņām izmērs),
* iegūst 0 punktus citādi.

Par katru testu, kur kopa ar visām tukšām rūtiņām *nav* regulārs stadions, jūsu risinājums:
* iegūst pilnus punktus, ja atgriež pareizu atbildi,
* iegūst 0 punktus, ja atgriež kopas ar visām tukšām rūtiņām izmēru,
* iegūst 25% punktu, ja atgriež jebkuru citu vērtību.

Katra apakšuzdevuma vērtējums ir mazākais punktu skaits pāri testiem šajā apakšuzdevumā.

## Piemēra vērtētājprogramma
 
Piemēra vērtētājprogramma nolasa ievadu šādā formātā:
 
* $1.$ rinda: $N$
* $(2 + i)$-tā ($0 \le i \lt N$) rinda:  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$

Piemēra vērtētājprogramma izdrukā jūsu atbildi šādā formātā:
* $1.$ rinda: `biggest_stadium` atgrieztā vērtība
