# 足球場
Nagyerdő是位於Debrecen市的一個正方形森林，它可以建模為一個$N \times N$的單元格網格。
網格的行從北到南編號為$0$到$N - 1$，網格的列從西到東編號為$0$到$N - 1$。
我們將位於網格的第$r$行和第$c$列的單元格稱為$(r, c)$單元格。

在森林中，每個單元格要麼是**空的**，要麼包含一棵**樹**。
森林中至少有一個空的單元格。

DVSC，這座城市著名的體育俱樂部，計劃在森林中建造一座新的足球場。
一個大小為$s$（其中$s \ge 1$）的足球場是一個由$s$個*不同的空*單元格$(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$所組成的集合。

正式的意思是:
- 對於每個 $i$ 從 $0$ to $s - 1$（包含） 單元格 $(r_i, c_i)$ 是空的,
- 對於每個 $i, j$ 使 $0 \le i \lt j \lt s$, 至少有一個 $r_i \neq r_j$ 和 $c_i \neq c_j$ 成立。


足球比賽使用一個在足球場的單元格之間移動的球。
**直接踢球**定義為以下兩種操作之一：
* 將球從單元格$(r,a)$移動到單元格$(r,b)$（$0 \le r,a,b \lt N, a \ne b$），其中足球場包含行$r$中單元格$(r,a)$和$(r,b)$之間的*所有*單元格。形式上，
 - 如果$a \lt b$，則足球場應包含每個滿足$a \le k \le b$的單元格$(r,k)$，
 - 如果$a \gt b$，則足球場應包含每個滿足$b \le k \le a$的單元格$(r,k)$。
* 將球從單元格$(a,c)$移動到單元格$(b,c)$（$0 \le c,a,b \lt N, a \ne b$），其中足球場包含列$c$中單元格$(a,c)$和$(b,c)$之間的*所有*單元格。形式上，
 - 如果$a \lt b$，則足球場應包含每個滿足$a \le k \le b$的單元格$(k,c)$，
 - 如果$a \gt b$，則足球場應包含每個滿足$b \le k \le a$的單元格$(k,c)$。

如果可以在足球場包含的任何單元格之間用最多$2$次直接踢球將球移動到另一個足球場包含的單元格，則該足球場是**合規**的。
注意，任何大小為$1$的足球場都是合規的。
​
例如，考慮一個大小為$N = 5$的森林，其中單元格$(1,0)$和$(4,2)$包含樹，其他每個單元格都是空的。
下圖顯示了三個可能的足球場。黑色單元格為樹的單元格，有條紋的單元格為足球場。
 
![Stadiums](soccer-regular.png "650")
​
左側的足球場是合規的。然而，中間的足球場是不合規的，因為至少需要$3$次直接踢球才能將球從單元格$(4,1)$移動到$(4,3)$。右側的足球場也不是合規的，因為使用直接踢球無法將球從單元格$(3,0)$移動到$(1,3)$。
​
體育俱樂部希望建造一個最大的合規足球場。
你的任務是找到在森林中可以建造的最大合規足球場$s$的值。

## 編程實現細節

你應該編寫以下的子程序。

```
int biggest_stadium(int N, int[][] F)
```

* $N$：森林的大小。
* $F$：長度為$N$的數組，包含描述森林中單元格的數組。對於每個$0 \le r \lt N$和$0 \le c \lt N$的$r$和$c$，$F[r][c] = 0$表示單元格$(r, c)$是空的，否則$F[r][c] = 1$表示它包含一棵樹。
* 這個子程序應該返回可以在森林中建造的最大合規足球場的大小。
* 對於每個測試樣例，這個子程序只被調用一次。

## 範例

考慮以下的調用：

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

在這個範例中，森林顯示在左邊，大小為 $20$ 的合規球場顯示在右邊的下圖中：

![範例](soccer-example.png "400")

由於沒有大小為 $21$ 或更大的合規體育場，該程序應該返回 $20$。

## 約束條件

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$（對於每個 $i$ 和 $j$，滿足 $0 \le i \lt N$ 和 $0 \le j \lt N$）
* 森林中至少有一個空單元格。換句話來說，對于 $0 \le i \lt N$ 及 $0 \le j \lt N$，
$F[i][j] = 0$

## 子任務

1. （6 分）最多只有一個包含樹的單元格。
2. （8 分）$N \le 3$
3. （22 分）$N \le 7$ 
4. （18 分）$N \le 30$ 
5. （16 分）$N \le 500$
6. （30 分）沒有額外的約束。

在每個子任務中，如果你的程序能夠正確判斷出那個由*所有*空單元格組成的集是不是一個正規的球場，你可以獲得這個子任務的25%的分數。

更準確地說，對於每個由所有空單元組成的集合是正規球場的測試樣例，您的解決方案：
* 如果返回正確答案（即所有空單元格組成的集合的大小），則獲得滿分。
* 否則，不得分。

對於每個由所有空單元組成的集合不是正規體育場的測試樣例，您的解決方案：
* 如果返回正確答案，則獲得滿分。
* 如果返回所有空單元格組成的集合的大小，則不得分。
* 如果返回任何其他值，則獲得滿分的 25%。

每個子任務的得分是該子任務中測試用例的得分的最小值。

## 樣例評分器

樣例評分器按照以下格式讀取輸入：

* 第 $1$ 行：$N$
* 第 $2 + i$ 行（$0 \le i \lt N$）：$F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$

樣例評分器按照以下格式打印你的答案：
* 第 $1$ 行：`biggest_stadium` 的返回值

