# Фудбалски стадион

Nagyerdő е шума со квадратен облик, лоцирана во градот Дебрецин, која може да се моделира како $N \times N$ квадратна мрежа (матрица) од ќелии.
Редиците на матрицата се нумерирани со целите броеви од $0$ до $N - 1$, од север кон југ, а колоните се нумерирани со целите броеви од $0$ до $N - 1$, од запад кон исток.
Ќелијата која се наоѓа во редицата $r$ и колоната $c$ од матрицата ќе ја означуваме со  $(r, c)$.

Во шумата, секоја ќелија е или **празна**, или пак содржи **дрво**.
Барем една ќелија во шумата е празна.

DVSC, познатиот спортски клуб на градот Дебрецин, планира да изгради нов фудбалски стадион во шумата.
Стадион со големина $s$ (каде $s \ge 1$) претставува множество од $s$ *различни празни* ќелии $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$.
Формално, ова значи дека
- за секое $i$ од $0$ до $s - 1$, вклучително, ќелијата $(r_i, c_i)$ е празна,
- за секои $i, j$ така што $0 \le i \lt j \lt s$, важи барем едно од следниве: $r_i \neq r_j$ и $c_i \neq c_j$.

Фудбалот се игра користејќи топка која се движи низ клетките од стадионот.
**Прав шут** се дефинира како една од следните две акции:
* Придвижи ја топката од ќелијата $(r,a)$ во ќелијата $(r,b)$ ($0 \le r,a,b \lt N, a \ne b$), каде што стадионот ги содржи *сите* ќелии помеѓу ќелијата $(r,a)$ и ќелијата $(r,b)$ во редицата $r$. Формално,
 - ако $a \lt b$ тогаш стадионот треба да ја содржи ќелијата $(r,k)$ за секое $k$ такво што $a \le k \le b$,
 - ако $a \gt b$ тогаш стадионот треба да ја содржи ќелијата $(r,k)$ за секое $k$ такво што $b \le k \le a$.
* Придвижи ја топката од ќелијата $(a,c)$ во ќелијата $(b,c)$ ($0 \le c,a,b \lt N, a \ne b$), каде што стадионот ги содржи *сите* ќелии помеѓу ќелијата $(a,c)$ и ќелијата $(b,c)$ во колоната $c$. Формално,
 - ако $a \lt b$ тогаш стадионот треба да ја содржи ќелијата $(k,c)$ за секое $k$ такво што $a \le k \le b$,
 - ако $a \gt b$ тогаш стадионот треба да ја содржи ќелијата $(k,c)$ за секое $k$ такво што $b \le k \le a$.

Еден стадион е **регуларен** ако е возможно топката да се придвижи од која било ќелија што е содржана во стадионот во која било друга ќелија што е содржана во стадионот со најмногу $2$ прави шутови.
Да забележиме дека секој стадион со големина $1$ е регуларен.

На пример, да разгледаме една шума со големина $N = 5$, каде што ќелиите $(1,0)$ и $(4,2)$ содржат дрва, а сите останати ќелии се празни.
На сликата подолу се прикажани три можни стадиони. Ќелиите со дрва се затемнети, а ќелиите содржани во стадионот се шрафирани.
 
![Stadiums](soccer-regular.png "650")

Стадионот лево е регуларен. Но, стадионот во средината не е регуларен, бидејќи потребни се најмалку $3$ прави шута за топката да се придвижи од ќелијата $(4,1)$ во ќелијата $(4,3)$. Стадионот десно исто така не е регуларен, бидејќи е невозможно топката да се придвижи од ќелијата $(3,0)$ во ќелијата $(1,3)$ користејќи прави шутови.

Спортскиот клуб сака да изгради регуларен стадион кој ќе биде колку што е можно поголем.
Ваша задача е да ја најдете максималната вредност за $s$ таква што да постои регуларен стадион со големина $s$ во шумата.

## Имплементациски детали

Треба да ја имплементирате следната процедура:

```
int biggest_stadium(int N, int[][] F)
```

* $N$: големината на шумата.
* $F$: низа со должина $N$ која содржи низи со должина $N$, што ги опишуваат ќелиите во шумата. За секои $r$ и $c$ така што $0 \le r \lt N$ и $0 \le c \lt N$, $F[r][c] = 0$ значи дека ќелијата $(r, c)$ е празна, а $F[r][c] = 1$ значи дека таа содржи дрво.
* Оваа процедура треба да ја враќа максималната големина на регуларен стадион кој може да се изгради во шумата.
* Оваа процедура се повикува точно еднаш за секој тест случај.

## Пример

Да го разгледаме следниот повик:

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

Во овој пример, шумата е прикажана на сликата лево, а на сликата десно е прикажан регуларен стадион со големина $20$:

![Example](soccer-example.png "400")

Бидејќи не постои регуларен стадион со големина $21$ или повеќе, процедурата треба да врати вредност $20$.

## Ограничувања

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (за секои $i$ и $j$ така што $0 \le i \lt N$ и $0 \le j \lt N$)
* Постои барем една празна ќелија во шумата. Со други зборови, $F[i][j] = 0$ за некое $0 \le i \lt N$ и $0 \le j \lt N$.

## Подзадачи

1. (6 поени) Постои најмногу една ќелија што содржи дрво.
2. (8 поени) $N \le 3$
3. (22 поени) $N \le 7$ 
4. (18 поени) $N \le 30$ 
5. (16 поени) $N \le 500$
6. (30 поени) Без дополнителни ограничувања.

Во секоја подзадача, можете да добиете 25% од поените на таа подзадача ако вашата програма точно одредува дали множеството што се состои од *сите* празни ќелии претставува регуларен стадион.

Попрецизно, за секој тест случај во кој множеството што се состои од сите празни ќелии претставува регуларен стадион, вашето решение:
* ги добива сите поени ако го враќа точниот одговор (којшто е големината на множеството што се состои од сите празни ќелии).
* во спротивно добива 0 поени.

За секој тест случај во кој множеството множеството што се состои од сите празни ќелии *не претставува* регуларен стадион, вашето решение:
* ги добива сите поени ако го враќа точниот одговор.
* добива 0 поени ако ја враќа големината на множеството што се состои од сите празни ќелии.
* добива 25% од поените ако враќа која било друга вредност.

Резултатот (освоените поени) за секоја подзадача се пресметува како минимум од поените за тест случаите во таа подзадача.

## Пример-оценувач

Пример-оценувачот го чита влезот во следниот формат: 
 
* линија $1$: $N$
* линија $2 + i$  ($0 \le i \lt N$):  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
Пример-оценувачот го печати вашиот одговор во следниот формат:
* линија $1$: повратната вредност на процедурата `biggest_stadium`
