# Фудбалски стадион

Сурчин је шума квадратног облика која се налази поред Београда, коју можемо представити као решетку од $N \times N$ поља.
Редови су нумерисани од $0$ до $N - 1$ од севера према југу, а колоне су нумерисане од $0$ до $N - 1$ од запада према истоку. Именујемо поље које се налази у реду $r$ и колони $c$ решетке као поље $(r, c)$.

У свакој шуми, свако од поља је или **празно** или садржи **дрво**.
Барем једно поље је празно.

ФМП, познат спортски клуб из предграђа Београда, планира да изгради нови фудбалски стадион у Сурчинској шуми. Стадион величине $s$ (где је $s \ge 1$) је скуп од $s$ *различитих празних* поља $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$.
То значи, за свако $i$ од $0$ до $s - 1$, поље $(r_i, c_i)$ је празно, и за свако $j$ такво да $i \lt j \lt s$, $r_i \neq r_j$ или $c_i \neq c_j$ важи.

Фудбал се игра користећи лопту која може да се помера по пољима стадиона.

**Шпиц-шут** је дефинисан као једна од две наведене акције:
* Померање лопте од поља $(r,a)$ у поље $(r,b)$ ($0 \le r,a,b \lt N, a \ne b$), где стадион садржи *сва* поља између почетног поља $(r,a)$ и крајњег поља $(r,b)$ у реду $r$. Формално,
 - ако је $a \lt b$ онда стадион мора да садржи $(r,k)$ за свако $k$ тако да важи $a \le k \le b$,
 - ако је $a \gt b$ онда стадион мора да садржи $(r,k)$ за свако $k$ тако да важи $b \le k \le a$.
* Померање лопте од поља $(a,c)$ у поље $(b,c)$ ($0 \le c,a,b \lt N, a \ne b$), где стадион садржи *сва* поља између почетног поља $(a,c)$ и крајњег поља $(b,c)$ у колони $c$. Формално,
 - ако је $a \lt b$ онда стадион мора да садржи $(k,c)$ за свако $k$ тако да важи $a \le k \le b$,
 - ако је $a \gt b$ онда стадион мора да садржи $(k,c)$ за свако $k$ тако да важи $b \le k \le a$.

Стадион зовемо **правилан** ако је могуће да се лопта помери из било ког поља које је садржано у стадиону до било ког поља које је садржано у стадиону у **највише $2$ шпиц-шута**.
Приметимо да је било који стадион величине $1$ правилан.

На пример, посматрајмо шуму величине $N = 5$, са пољима $(1,0)$ и $(4,2)$ које садрже дрвеће и свим осталим пољима која су празна.
На слици испод можемо да видимо три стадиона. Поља са дрвећем су затамњена, а поља која су део стадиона су шрафирана (и плава).
 
![Stadiums](soccer-regular.png "650")

Стадион лево је правилан. Стадион у средини није правилан, зато што су најмање $3$ шпиц-шута потребна да се лопта помери са поља $(4,1)$ на поље $(4,3)$. Стадион десно такође није правилан, зато што је немогуће померити лопту са поља $(3,0)$ на поље $(1,3)$ шпиц-шутевима.

ФМП жели да направи стадион тако да буде што је могуће већи, јер нико не може да им забрани колики стадион желе да направе, а такође и да буде правилан. Ваш задатак је да нађете највећу вредност $s$ такву да се правилан стадион величине $s$ може направити у Сурчинској шуми.

## Детаљи имплементације

Треба да имплементирате следећу процедуру.

```
int biggest_stadium(int N, int[][] F)
```

* $N$: величина шуме.
* $F$: низ дужине $N$ који садржи низове дужине $N$, који описује поља у шуми. За свако $r$ и $c$ тако да је  $0 \le r \lt N$ и $0 \le c \lt N$, $F[r][c] = 0$ значи да је поље $(r, c)$ празно, а $F[r][c] = 1$ значи да садржи дрво.
* Ова процедура треба да врати максималну величину правилног стадиона који може да се направи у шуми.
* Ова процедура ће бити позвана тачно једном за сваки тест пример.

## Пример

Посматрајмо следећи позив функције:

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

У овом примеру, шума је приказана на слици лево и правилан стадион величине $20$ је приказан на слици десно:

![Example](soccer-example.png "400")

Обзиром на то да не постоји правилан стадион са величином $21$ и више, процедура треба да врати $20$.

## Ограничења

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (за свако $i$ и $j$ тако да важи $0 \le i \lt N$ и $0 \le j \lt N$)
* Постоји бар једно празно поље у шуми. Другим речима, $F[i][j] = 0$ за неко $0 \le i \lt N$ и $0 \le j \lt N$.

## Подзадаци

1. (6 поена) Постоји највише једно поље које садржи дрво.
2. (8 поена) $N \le 3$
3. (22 поена) $N \le 7$ 
4. (18 поена) $N \le 30$ 
5. (16 поена) $N \le 500$
6. (30 поена) Без додатних ограничења.

У сваком подзадатку, можете да освојите 25% поена за тај подзадатак уколико ваш програм тачно одреди да ли скуп који се састоји од *свих* празних поља представља правилан стадион.

Презицније, за сваки тест пример у коме је скуп свих празних поља правилан стадион, ваше решење:
* добија све поене уколико врати тачан одговор (који је величина скупа који садржи сва празна поља).
* у супротном, добија 0 поена.

За сваки тест пример у коме скуп свих празних поља *није* правилан стадион, ваше решење:
* добија све поене уколико врати тачан одговор.
* добија 0 поена уколико врати величину скупа који се састоји од свих празних поља.
* добија 25% поена уколико врати било који другу вредност.

Број поена освојен у сваком подзадатку је минимум поена који су освојили тест примери у том подзадатку.

## Пример оцењивача (sample grader)

Пример оцењивача учитава улаз у следећем формату: 
 
* линија $1$: $N$
* линија $2 + i$ ($0 \le i \lt N$): $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
Оцењивач исписује ваше решење у следећем формату:
* линија $1$: повратна вредност функције `biggest_stadium`
