# Futbalový štadión
V meste Debrecín sa nachádza les Nagyerdő, ktorý môžeme popísať štvorcovou mriežkou $N \times N$ ..
Riadky sú očíslované od $0$ do $N - 1$ zo severu na juh a stĺpce sú očíslované od  $0$ do $N - 1$ zo západu na východ.
Políčko nachádzajúce sa v riadku $r$ a stĺpci $c$ mriežky budeme označovať $(r, c)$.

V lese je každé políčko buď **prázdne** alebo obsahuje **strom**. Aspoň jedno políčko  v celom lese je prázdne.

Známy mestský športový klub DVSC plánuje v tomto lese postaviť nový futbalový štadión.
**Štadión** veľkosti $s$ (kde $s \ge 1$) je množina $s$ *navzájom rôznych prázdnych* buniek $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$.
Formálne:
- pre všetky $i$ od $0$ do $s - 1$ (vrátane), bunka $(r_i, c_i)$ je prázdna,
- pre každé $i, j$ také, že $0 \le i \lt j \lt s$, platí aspoň jedna z nerovností $r_i \neq r_j$ a $c_i \neq c_j$.

Futbal sa hraje s loptou, ktorá sa pohybuje po políčkach štadiónu.
**Priamy kop** je definovaný ako jedna z dvoch nasledujúcich možností:
* Presun lopty z políčka $(r,a)$ na políčko $(r,b)$ ($0 \le r,a,b \lt N, a \ne b$), kde štadión obsahuje *všetky* políčka medzi políčkami $(r,a)$ a $(r,b)$ v riadku $r$. Formálne,
 - ak $a \lt b$, tak štadión musí obsahovať políčka $(r,k)$ pre každé $k$ také, že $a \le k \le b$,
 - ak $a \gt b$, tak štadión musí obsahovať políčka $(r,k)$ pre každé $k$ také, že $b \le k \le a$.
* Presun lopty z políčka $(a,c)$ na políčko $(b,c)$ ($0 \le c,a,b \lt N, a \ne b$), kde štadión obsahuje *všetky* políčka medzi políčkami $(a,c)$ a $(b,c)$ v stĺpci $c$. Formálne,
 - ak $a \lt b$, tak štadión musí obsahovať políčka $(k,c)$ pre každé $k$ také, že $a \le k \le b$,
 - ak $a \gt b$, tak štadión musí obsahovať políčka  $(k,c)$ pre každé $k$ také, že $b \le k \le a$.

Štadión voláme **regulárny**, ak je možné presunúť loptu z ľubovoľného políčka na iné políčko štadióna pomocou *najviac dvoch* priamych kopov.
Je zrejmé, že každý štadión veľkosti $1$ je regulárny.

Napríklad uvažujme les veľkosti $N = 5$ s políčkami $(1,0)$ a $(4,2)$ obsahujúcimi stromy. Všetky ostatné políčka sú prázdne.
Nasledovný obrázok ukazuje tri rôzne štadióny. Políčka so stromami sú čierne a políčka tvoriace štadión sú vyšrafované.
 
![Stadiums](soccer-regular.png "650")

Štadión vľavo je regulárny. Štadión v strede nie je regulárny, lebo na presun lopty z políčka $(4,1)$ na políčko  $(4,3)$ sú potrebné aspoň $3$ priame kopy. Ani štadión vpravo nie je regulárny, lebo vôbec nie je možné dostať loptu z políčka $(3,0)$ na políčko $(1,3)$ priamymi kopmi.

Športový klub chce vybudovať čo najväčší štadión. Vašou úlohou je teda nájsť maximálnu veľkosť $s$ regulárneho štadióna v zadanom lese.

## Implementačné detaily

Vašou úlohou je naprogramovať nasledujúcu funkciu:

```
int biggest_stadium(int N, int[][] F)
```

* $N$: veľkosť lesa
* $F$: pole dĺžky $N$ obsahujúce polia dĺžky $N$. Toto pole popisuje políčka lesa. Pre každé $r$ a $c$ také, že $0 \le r \lt N$ a $0 \le c \lt N$, $F[r][c] = 0$ znamená, že políčko $(r, c)$ je prázdne a  $F[r][c] = 1$ znamená, že obsahuje strom.
* Táto funkcia má vrátiť maximálnu veľkosť regulárneho štadióna, ktorý môže byť vybudovaný v zadanom lese.
* Táto funkcia je v každej testovacej sade zavolaná práve raz.

## Príklad

Uvažujme nižšie uvedené volanie:

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

V tomto príklade je les zobrazený na obrázku vľavo a vpravo je regulárny štadión veľkosti $20$:

![Example](soccer-example.png "400")

Keďže neexistuje regulárny štadión veľkosti $21$ alebo viac, tak vaša funkcia má vrátiť hodnotu $20$.

## Obmedzenia

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (pre každé $i$ a $j$ také, že $0 \le i \lt N$ a $0 \le j \lt N$)
* V lese sa nachádza aspoň jedno prázdne políčko. Teda, inými slovami, existuje $0 \le i \lt N$ a $0 \le j \lt N$ také, že $F[i][j] = 0$.

## Podúlohy

1. (06 bodov) V lese sa nachádza najviac jedno políčko so stromom.
2. (08 bodov) $N \le 3$
3. (22 bodov) $N \le 7$ 
4. (18 bodov) $N \le 30$ 
5. (16 bodov) $N \le 500$
6. (30 bodov) bez ďalších obmedzení

V každej podúlohe môžete získať 25% bodov podúlohy v prípade, že váš program počíta korektne v prípade, že všetky prázdne políčka tvoria regulárny štadión.

Presnejšie v každej testovacej sade, kde je možné vybudovať regulárny štadión na všetkých prázdnych políčkach vaše riešenie získa:
* plný počet bodov, ak dá správnu odpoveď (teda vráti veľkosť množiny všetkých prázdnych políčok)
* 0 bodov v opačnom prípade.

V každej testovacej sade, kde nie je možné vybudovať regulárny štadión na všetkých prázdnych políčkach vaše riešeni získa:
* plný počet bodov, ak dá správnu odpoveď
* 0 bodov, ak vráti veľkosť množiny všetkých prázdnych políčok
* 25% bodov, ak vráti ľubovoľnú inú hodnotu.

Body za každú podúlohu sú rovné minimu z počtov bodov, ktoré vaše riešenie získa za jednotlivé testovacie sady patriace do tejto podúlohy.

## Ukážkový testovač

Ukážkový testovač číta vstup vo formáte:
 
* riadok $1$: $N$
* riadok $2 + i$  ($0 \le i \lt N$):  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
Ukážkový testovač vypisuje odpoveď vo formáte:
* riadok $1$: návratová hodnota funkcie `biggest_stadium`
