
# 足球場 (Soccer Stadium)

Nagyerdő 是一座在 Debrecen 城市的方格狀森林，可以被 $N \times N$ 的方陣表示。 方陣中的列 (rows)，由北至南依序標上 $0$ 至 $N - 1$ 號；方陣中的行 (columns)，由西往東依序標上 $0$ 至 $N - 1$ 號。 方陣中位在第 $r$ 列和第 $c$ 行的方格稱作方格 $(r, c)$。

在這森林中，每一個方格的狀態是以下二擇一：(1) **空格**、(2)**種了一棵樹**。除此之外，這森林包含至少一個空格。

Debrecen 城市有一個知名的體育中心 DVSC，正在規劃如何在這森林蓋一座足球場。如果我們說一個球場的大小是 $s$ ($s \ge 1$)，那就意謂著該球場恰好是 $s$ 個位置不同的空格，我們用 $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$ 表示這 $s$ 個空格的位置。下面是嚴謹的描述，就是對於每一個介於 $0$ 至 $s-1$ 之間的 $i$ (包含邊界值 $0$ 和 $s-1$)，方格 $(r_i, s_i)$ 是空格，而且當 $0 \le i \lt j \lt s$，下面兩種條件至少有一個成立：$r_i \ne r_j$ 及 $c_i \ne c_j$。

足球的玩法是把一顆球在足球場涵蓋的格子中移來移去。下面的任一個動作，都視為一次 **直線踢**。

* 把球從方格 $(r, a)$ 移至方格 $(r, b)$ ($0 \le r, a, b < N, a \ne b$)，這動作可以做的條件是足球場涵蓋第 $r$ 列中，介於 $(r, a)$ 和 $(r, b)$ 中的所有方格。下面是嚴謹的描述：
	-  如果 $a \lt b$，足球場就必須涵蓋方格 $(r, k)$ ($a \le k \le b$)，
	-  如果 $a \gt b$，足球場就必須涵蓋方格 $(r, k)$ ($b \le k \le a$)。
* 把球從方格 $(a, c)$ 移至方格 $(b, c)$ ($0 \le c, a, b < N, a \ne b$)，這動作可以做的條件是足球場涵蓋第 $c$ 行中，介於 $(a, c)$ 和 $(b, c)$ 中的所有方格。下面是嚴謹的描述：
	-  如果 $a \lt b$，足球場就必須涵蓋方格 $(k, c)$ ($a \le k \le b$)，
	-  如果 $a \gt b$，足球場就必須涵蓋方格 $(k, c)$ ($b \le k \le a$)。

我們稱一個足球場 **正規** 的條件是：從足球場任挑兩個方格，一顆球可以透過至多兩次直線踢從其中一個方格被移動至另外一個方格。大小為 $1$ 的足球場也被視為正規。

這邊我們舉個例子。考慮一個 $N = 5$ 的森林，其中方格 $(1, 0)$ 和方格 $(4, 2)$ 都種了一棵樹，其他方格是空格。下圖有三個可能的足球場蓋法。有種樹的方格被塗黑，被足球場涵蓋的方格被畫斜線。
 
![Stadiums](soccer-regular.png "650")

上圖左邊的足球場是一個正規球場。但是，上圖中間的足球場不是正規，因為要把球從方格 $(4, 1)$ 移至方格 $(4, 3)$ 至少要三次直線踢。另外，上圖右邊的足球場也不是正規，因為沒有辦法透過直線踢把球從方格 $(3, 0)$ 移至方格 $(1, 3)$。

該體育中心想要蓋一座大小儘可能大的正規球場。你的任務就是找到最大的 $s$ 使得這森林能夠蓋出一個大小為 $s$ 的正規球場。


## 實作細節 (Implementation Details)

你被要求實作下面這個程序。

```
int biggest_stadium(int N, int[][] F)
```

* $N$: Nagyerdő 森林的大小參數。
* $F$: 一個 $N \times N$ 大小的方陣，用來描述 Nagyerdő 森林。描述的方式如下，當 $0 \le r < N$ 且 $0 \le c < N$，$F[r][c] = 0$ 代表方格 $(r, c)$ 是空格，$F[r][c] = 1$ 代表方格 $(r, c)$ 種了棵樹。
* 這程序需要回傳 Nagyerdő 森林能蓋出的最大正規球場的大小。
* 對於每一筆測資，你實作的程序會被呼叫恰好一次。

## 例子 (Example)

考慮你實作的程序被下面的方式呼叫。

```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

在這個例子，Nagyerdő 森林被畫在下圖左。其中一種 $20$ 格的正規球場蓋法，如下圖右。

![Example](soccer-example.png "400")

因為沒辦法蓋出 $21$ 格甚至更多格的正規球場，你實作的程序應該要輸出 $20$。

## 限制 (Constraints)

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (當 $0 \le i \lt N$ 且 $0 \le j \lt N$)
* Nagyerdő 森林涵蓋至少一格空格。換句話說，就是存在某對 $0 \le i \lt N$ 和 $0 \le j \lt N$ 使得$F[i][j] = 0$。 

## 子任務 (Subtasks)

1. (6 points) 至多有一格被種樹。
2. (8 points) $N \le 3$
3. (22 points) $N \le 7$ 
4. (18 points) $N \le 30$ 
5. (16 points) $N \le 500$
6. (30 points) 沒有額外限制。

對於每一組子任務，你的程序能拿 25% 部分分數的條件是，只要你的程序能夠正確判斷這件事：森林中的 *全部* 空格合起來是不是一個正規球場？ 

細節如下：當一筆測資中的全部空格合起來是一個正規球場，如果你輸出的答案：
* 是正確的 (也就是森林中空格的個數)，這筆拿全部分數。
* 不是正確的，這筆拿 0 分。

當一筆測資中的全部空格合起來不是一個正規球場，如果你輸出的答案：
* 是正確的，這筆拿全部分數。
* 輸出森林中的空格個數，這筆拿 0 分。
* 輸出和上面兩個選項不一樣的答案，這筆拿 25% 部分分數。

每一組子任務的得分，就是這組子任務中得分最低的一筆測資的得分。

## 範例評分程式 (Sample Grader)

範例評分程式的輸入格式：
 
* line $1$: $N$
* line $2 + i$  ($0 \le i \lt N$):  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$

範例評分程式的輸出格式：

* line $1$: `biggest_stadium` 的回傳值
