# Soccer Stadium

Nagyerdő là một khu rừng hình vuông nằm ở thành phố Debrecen, được mô hình hóa bằng một lưới ô vuông kích thước $N \times N$.
Các hàng được đánh số từ $0$ đến $N - 1$ từ bắc xuống nam và các cột của lưới được đánh số từ $0$ đến $N - 1$ từ tây sang đông.
Gọi ô nằm ở hàng $r$ và cột $c$ của lưới là ô $(r, c)$.

Trong khu rừng, mỗi ô hoặc là **trống** hoặc chứa một **cây**.
Ít nhất một ô trong khu rừng là trống.

DVSC, câu lạc bộ thể thao nổi tiếng của thành phố, đang có kế hoạch xây dựng một sân vận động bóng đá mới trong khu rừng.
Một sân vận động có kích thước $s$ (với $s \ge 1$) là một tập gồm $s$ ô *trống khác nhau* $(r_0, c_0), \ldots, (r_{s - 1}, c_{s - 1})$.
Một cách chính xác:
- với mỗi $i$ từ $0$ đến $s - 1$ (kể cả $0$ và $s - 1$) thì ô $(r_i, c_i)$ là trống,
- với mỗi $i, j$ mà $0 \le i \lt j \lt s$, ít nhất một trong hai điều kiện $r_i \neq r_j$ và $c_i \neq c_j$ thỏa mãn.

Bóng đá được chơi bằng cách sử dụng một quả bóng di chuyển qua các ô của sân vận động.
Một **cú sút thẳng** được định nghĩa là một trong hai hành động sau:
* Di chuyển quả bóng từ ô $(r,a)$ đến ô $(r,b)$ ($0 \le r,a,b \lt N, a \ne b$), trong đó sân vận động chứa *tất cả* các ô giữa hai ô $(r,a)$ và $(r,b)$ trong hàng $r$. Một cách chính xác,
  - nếu $a \lt b$ thì sân vận động phải chứa các ô $(r,k)$ với mỗi $k$ thỏa mãn $a \le k \le b$,
  - nếu $a \gt b$ thì sân vận động phải chứa các ô $(r,k)$ với mỗi $k$ thỏa mãn $b \le k \le a$.
* Di chuyển quả bóng từ ô $(a,c)$ đến ô $(b,c)$ ($0 \le c,a,b \lt N, a \ne b$), trong đó sân vận động chứa *tất cả* các ô giữa hai ô $(a,c)$ và $(b,c)$ trong cột $c$. Một cách chính xác,
  - nếu $a \lt b$ thì sân vận động phải chứa các ô $(k,c)$ với mỗi $k$ thỏa mãn $a \le k \le b$,
  - nếu $a \gt b$ thì sân vận động phải chứa các ô $(k,c)$ với mỗi $k$ thỏa mãn $b \le k \le a$.
 
Một sân vận động là **chuẩn** nếu có thể di chuyển quả bóng từ ô bất kì trong sân vận động đến bất kì ô nào khác trong sân vận động với tối đa $2$ cú sút thẳng.
Lưu ý rằng bất kỳ sân vận động nào có kích thước $1$ đều là sân vận động chuẩn.

Ví dụ: xét một khu rừng có kích thước $N = 5$, với các ô $(1,0)$ và $(4,2)$ chứa cây và mọi ô khác đều trống.
Hình dưới đây mô tả ba sân vận động. Các ô chứa cây có màu đen, các ô thuộc sân vận động có sọc.

![Stadiums](soccer-regular.png "650")

Sân vận động bên trái là chuẩn. Tuy nhiên, sân vận động ở
giữa không là chuẩn, vì cần ít nhất $3$ cú sút thẳng để di chuyển quả bóng từ
ô $(4,1)$ đến $(4,3)$. Sân vận động bên phải cũng không là chuẩn, vì không thể
di chuyển quả bóng từ ô $(3,0)$ đến $(1,3)$ bằng cách sử dụng các cú sút thẳng.

Câu lạc bộ thể thao muốn xây dựng một sân vận động chuẩn càng lớn càng tốt.
Nhiệm vụ của bạn là tìm giá trị lớn nhất của $s$ sao cho tồn tại một sân vận động chuẩn có kích thước $s$ trong khu rừng.
## Chi tiết cài đặt

Bạn cần cài đặt hàm sau:

```
int biggest_stadium(int N, int[][] F)
```

* $N$: kích thước khu rừng.
* $F$: một mảng kích thước $N$ mà mỗi phần tử là một mảng kích thước $N$, mô tả các ô trong khu rừng. Với mỗi $r$ và $c$ thỏa mãn $0 \le r \lt N$ và $0 \le c \lt N$, $F[r][c] = 0$ nghĩa là ô $(r, c)$ trống, và $F[r][c] = 1$ nghĩa là ô chứa cây.
* Hàm cần trả về kích thước lớn nhất của sân vận động chuẩn có thể xây dựng trong khu rừng.
* Hàm được gọi đúng một lần với mỗi test.

## Ví dụ

Xét lời gọi hàm sau:
```
biggest_stadium(5, [[0, 0, 0, 0, 0],  
                    [1, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 0, 0, 0], 
                    [0, 0, 1, 0, 0]])
```

Trong ví dụ này, khu rừng được mô tả ở bên trái và một sân vận động chuẩn với kích thước $20$ được mô tả ở bên phải của hình dưới đây:
![Example](soccer-example.png "400")

Do không có sân vận động chuẩn nào có kích thước $21$ hoặc lớn hơn nên hàm cần trả về $20$.

## Các ràng buộc

* $1 \le N \le  2\,000$
* $0 \le F[i][j] \le 1$ (với mỗi $i$ và $j$ sao cho $0 \le i \lt N$ và $0 \le j \lt N$)
* Có ít nhất một ô trống trong khu rừng. Nói cách khác, $F[i][j] = 0$ với một vài $0 \le i \lt N$ và $0 \le j \lt N$.

## Các subtask

1. (6 điểm) Có nhiều nhất một ô chứa cây.
2. (8 điểm) $N \le 3$
3. (22 điểm) $N \le 7$ 
4. (18 điểm) $N \le 30$ 
5. (16 điểm) $N \le 500$
6. (30 điểm) Không có ràng buộc nào thêm.


Trong mỗi subtask, bạn có thể đạt được 25% số điểm của subtask nếu chương trình của bạn trả lời chính xác liệu tập bao gồm *tất cả* các ô trống có phải là một sân vận động chuẩn hay không.

Chính xác hơn, đối với mỗi test trong đó tập bao gồm tất cả các ô trống là một sân vận động chuẩn, lời giải của bạn:
* nhận được điểm tối đa nếu trả về câu trả lời đúng (là kích thước của tập bao gồm tất cả các ô trống).
* nếu không sẽ được 0 điểm.

Đối với mỗi test trong đó tập bao gồm tất cả các ô trống *không phải* là một sân vận động chuẩn, lời giải của bạn:
* được điểm tối đa nếu trả lời đúng.
* được 0 điểm nếu trả về kích thước của tập bao gồm tất cả các ô trống.
* nhận được 25% số điểm nếu nó trả về bất kỳ giá trị nào khác.

Điểm cho mỗi subtask là điểm thấp nhất trong các test của subtask đó.

## Trình chấm mẫu

Trình chấm mẫu đọc dữ liệu vào theo định dạng sau:
 
* dòng $1$: $N$
* dòng $2 + i$  ($0 \le i \lt N$):  $F[i][0] \; F[i][1] \; \ldots \; F[i][N - 1]$
 
Trình chấm mẫu ghi kết quả của bạn theo định dạng sau:
* dòng $1$: giá trị trả về của hàm `biggest_stadium`
